/**
 * @fileoverview This rule checks for category in sparrow track event calls
 * @author Sachin Fernandes
 */
'use strict'; //------------------------------------------------------------------------------
// Rule Definition
//------------------------------------------------------------------------------

function isValidType(argument, type) {
  return typeof argument !== 'undefined' && argument.type === type;
}

function hasKeyInObject(object, keyToFind) {
  // we want to capture both {'category': 'value'} and {category: 'value'}
  // When wrapped in quotes the key becomes a Literal and doesn't have a name but has a value
  return object.properties.some(function (property) {
    return property.key.name === keyToFind || property.key.value;
  });
}

function isTrackFunction(callee) {
  return (// We need to handle both track() and sparrow.track()
    callee.name === 'track' || callee.property && callee.property.name === 'track'
  );
}

module.exports = {
  meta: {
    docs: {
      description: 'This rule checks for recommended best practices for sparrow',
      category: 'Possible Errors',
      recommended: false
    },
    fixable: null,
    schema: [],
    messages: {
      missing: 'Missing category in sparrow track call'
    }
  },
  create: function create(context) {
    //----------------------------------------------------------------------
    // Public
    //----------------------------------------------------------------------
    return {
      CallExpression: function CallExpression(node) {
        if (isTrackFunction(node.callee)) {
          var arg1 = node.arguments[0];
          var arg2 = node.arguments[1];

          if (isValidType(arg1, 'Literal') && isValidType(arg2, 'ObjectExpression')) {
            if (!hasKeyInObject(arg2, 'category')) {
              context.report({
                node: node,
                loc: node.loc,
                messageId: 'missing'
              });
            }
          } else {
            context.report({
              node: node,
              loc: node.loc,
              messageId: 'missing'
            });
          }
        }
      }
    };
  }
};